@@ -1,5 +1,5 @@
 /* GDA library
- * Copyright (C) 1998 - 2008 The GNOME Foundation.
+ * Copyright (C) 1998 - 2009 The GNOME Foundation.
  *
  * AUTHORS:
  *	Rodrigo Moya <rodrigo@gnome-db.org>
@@ -22,7 +22,7 @@
  */
 
 #include <string.h>
-#include <glib/garray.h>
+#include <glib.h>
 #include <libgda/gda-data-model-array.h>
 #include <glib/gi18n-lib.h>
 #include <libgda/gda-data-model.h>
@@ -58,7 +58,7 @@ static void gda_data_model_array_get_property (GObject *object, guint prop_id, G
 static GObjectClass *parent_class = NULL;
 
 /* GdaDataModel interface */
-static void                 gda_data_model_array_data_model_init (GdaDataModelClass *iface);
+static void                 gda_data_model_array_data_model_init (GdaDataModelIface *iface);
 static gint                 gda_data_model_array_get_n_rows      (GdaDataModel *model);
 static gint                 gda_data_model_array_get_n_columns   (GdaDataModel *model);
 static GdaColumn           *gda_data_model_array_describe_column (GdaDataModel *model, gint col);
@@ -82,7 +82,7 @@ static gboolean             gda_data_model_array_get_notify      (GdaDataModel *
  */
 
 static void
-gda_data_model_array_data_model_init (GdaDataModelClass *iface)
+gda_data_model_array_data_model_init (GdaDataModelIface *iface)
 {
         iface->i_get_n_rows = gda_data_model_array_get_n_rows;
         iface->i_get_n_columns = gda_data_model_array_get_n_columns;
@@ -370,7 +370,7 @@ gda_data_model_array_copy_model (GdaDataModel *src, GError **error)
 /**
  * gda_data_model_array_get_row
  * @model: a #GdaDataModelArray object
- * @row:
+ * @row: row number (starting from 0)
  * @error: a place to store errors, or %NULL
  *
  * Get a pointer to a row in @model
@@ -383,9 +383,13 @@ gda_data_model_array_get_row (GdaDataModelArray *model, gint row, GError **error
 	g_return_val_if_fail (GDA_IS_DATA_MODEL_ARRAY (model), NULL);
 
 	if (row >= model->priv->rows->len) {
-		g_set_error (error, GDA_DATA_MODEL_ERROR, GDA_DATA_MODEL_ROW_OUT_OF_RANGE_ERROR,
-			     _("Row %d out of range (0-%d)"), row,
-			     model->priv->rows->len- 1);
+		if (model->priv->rows->len > 0)
+			g_set_error (error, GDA_DATA_MODEL_ERROR, GDA_DATA_MODEL_ROW_OUT_OF_RANGE_ERROR,
+				     _("Row %d out of range (0-%d)"), row,
+				     model->priv->rows->len - 1);
+		else
+			g_set_error (error, GDA_DATA_MODEL_ERROR, GDA_DATA_MODEL_ROW_OUT_OF_RANGE_ERROR,
+				     _("Row %d not found (empty data model)"), row);
 		return NULL;
 	}
 
@@ -521,13 +525,17 @@ gda_data_model_array_get_value_at (GdaDataModel *model, gint col, gint row, GErr
 
 	if (amodel->priv->rows->len == 0) {
 		g_set_error (error, GDA_DATA_MODEL_ERROR, GDA_DATA_MODEL_ROW_NOT_FOUND_ERROR,
-			     _("No row in data model"));
+			      "%s", _("No row in data model"));
 		return NULL;
 	}
 
 	if (row >= amodel->priv->rows->len) {
-		g_set_error (error, GDA_DATA_MODEL_ERROR, GDA_DATA_MODEL_ROW_OUT_OF_RANGE_ERROR,
-			     _("Row %d out of range (0-%d)"), row, amodel->priv->rows->len - 1);
+		if (amodel->priv->rows->len > 0)
+			g_set_error (error, GDA_DATA_MODEL_ERROR, GDA_DATA_MODEL_ROW_OUT_OF_RANGE_ERROR,
+				     _("Row %d out of range (0-%d)"), row, amodel->priv->rows->len - 1);
+		else
+			g_set_error (error, GDA_DATA_MODEL_ERROR, GDA_DATA_MODEL_ROW_OUT_OF_RANGE_ERROR,
+				     _("Row %d not found (empty data model)"), row);
 		return NULL;
 	}
 
@@ -542,11 +550,14 @@ gda_data_model_array_get_value_at (GdaDataModel *model, gint col, gint row, GErr
 		GValue *field;
 
 		field = gda_row_get_value (fields, col);
-		return (const GValue *) field;
+		if (gda_row_value_is_valid (fields, field))
+			return (const GValue *) field;
+		else
+			return NULL;
 	}
 	else {
 		g_set_error (error, GDA_DATA_MODEL_ERROR, GDA_DATA_MODEL_ROW_NOT_FOUND_ERROR,
-			     _("Data model has no data"));
+			      "%s", _("Data model has no data"));
 		return NULL;
 	}
 }
@@ -601,13 +612,17 @@ gda_data_model_array_set_value_at (GdaDataModel *model, gint col, gint row,
 
 	if (amodel->priv->read_only) {
 		g_set_error (error, GDA_DATA_MODEL_ERROR, GDA_DATA_MODEL_ACCESS_ERROR,
-                             _("Attempting to modify a read-only data model"));
+                              "%s", _("Attempting to modify a read-only data model"));
                 return FALSE;
         }
 
 	if (row > amodel->priv->rows->len) {
-		g_set_error (error, GDA_DATA_MODEL_ERROR, GDA_DATA_MODEL_VALUES_LIST_ERROR,
-			     _("Row %d out of range (0-%d)"), row, amodel->priv->rows->len);
+		if (amodel->priv->rows->len > 0)
+			g_set_error (error, GDA_DATA_MODEL_ERROR, GDA_DATA_MODEL_VALUES_LIST_ERROR,
+				     _("Row %d out of range (0-%d)"), row, amodel->priv->rows->len - 1);
+		else
+			g_set_error (error, GDA_DATA_MODEL_ERROR, GDA_DATA_MODEL_ROW_OUT_OF_RANGE_ERROR,
+				     _("Row %d not found (empty data model)"), row);
                 return FALSE;
         }
 
@@ -641,13 +656,13 @@ gda_data_model_array_set_values (GdaDataModel *model, gint row, GList *values, G
 
 	if (amodel->priv->read_only) {
 		g_set_error (error, GDA_DATA_MODEL_ERROR, GDA_DATA_MODEL_ACCESS_ERROR,
-                             _("Attempting to modify a read-only data model"));
+                              "%s", _("Attempting to modify a read-only data model"));
                 return FALSE;
         }
 
         if (g_list_length (values) > gda_data_model_get_n_columns (model)) {
                 g_set_error (error, GDA_DATA_MODEL_ERROR, GDA_DATA_MODEL_VALUES_LIST_ERROR,
-                             _("Too many values in list"));
+                              "%s", _("Too many values in list"));
                 return FALSE;
         }
 
@@ -680,13 +695,13 @@ gda_data_model_array_append_values (GdaDataModel *model, const GList *values, GE
 	
         if (amodel->priv->read_only) {
 		g_set_error (error, GDA_DATA_MODEL_ERROR, GDA_DATA_MODEL_ACCESS_ERROR,
-                             _("Attempting to modify a read-only data model"));
+                              "%s", _("Attempting to modify a read-only data model"));
                 return FALSE;
         }
 
 	if (g_list_length ((GList *) values) > amodel->priv->number_of_columns) {
                 g_set_error (error, GDA_DATA_MODEL_ERROR, GDA_DATA_MODEL_VALUES_LIST_ERROR,
-                             _("Too many values in list"));
+                              "%s", _("Too many values in list"));
                 return FALSE;
         }
 
@@ -715,7 +730,7 @@ gda_data_model_array_append_row (GdaDataModel *model, GError **error)
 
 	if (amodel->priv->read_only) {
 		g_set_error (error, GDA_DATA_MODEL_ERROR, GDA_DATA_MODEL_ACCESS_ERROR,
-                             _("Attempting to modify a read-only data model"));
+                              "%s", _("Attempting to modify a read-only data model"));
                 return FALSE;
         }
 
@@ -740,7 +755,7 @@ gda_data_model_array_remove_row (GdaDataModel *model, gint row, GError **error)
 	}
 
 	g_set_error (error, GDA_DATA_MODEL_ERROR, GDA_DATA_MODEL_ROW_NOT_FOUND_ERROR,
-		     _("Row not found in data model"));
+		      "%s", _("Row not found in data model"));
 	return FALSE;
 }
 
