@@ -138,6 +138,14 @@ gda_server_operation_class_init (GdaServerOperationClass *klass)
 	parent_class = g_type_class_peek_parent (klass);
 
 	/* signals */
+	/**
+	 * GdaServerOperation::sequence-item-added
+	 * @op: the #GdaServerOperation
+	 * @seq_path: the path to the new sequence item
+	 * @item_index: the index (starting from 0) of the new sequence item in the sequence
+	 *
+	 * Gets emitted whenever a new sequence item (from a sequence template) has been added
+	 */
 	gda_server_operation_signals[SEQUENCE_ITEM_ADDED] =
 		g_signal_new ("sequence-item-added",
 			      G_TYPE_FROM_CLASS (object_class),
@@ -146,6 +154,14 @@ gda_server_operation_class_init (GdaServerOperationClass *klass)
 			      NULL, NULL,
 			      _gda_marshal_VOID__STRING_INT, G_TYPE_NONE,
 			      2, G_TYPE_STRING, G_TYPE_INT);
+	/**
+	 * GdaServerOperation::sequence-item-remove
+	 * @op: the #GdaServerOperation
+	 * @seq_path: the path to the sequence item to be removed
+	 * @item_index: the index (starting from 0) of the sequence item in the sequence
+	 *
+	 * Gets emitted whenever a sequence item is about to be removed
+	 */
 	gda_server_operation_signals[SEQUENCE_ITEM_REMOVE] =
 		g_signal_new ("sequence-item-remove",
 			      G_TYPE_FROM_CLASS (object_class),
@@ -165,18 +181,20 @@ gda_server_operation_class_init (GdaServerOperationClass *klass)
 	object_class->get_property = gda_server_operation_get_property;
 
 	g_object_class_install_property (object_class, PROP_CNC,
-					 g_param_spec_object ("connection", NULL, NULL, 
+					 g_param_spec_object ("connection", NULL, "Connection to use", 
 							      GDA_TYPE_CONNECTION,
 							      G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
 	g_object_class_install_property (object_class, PROP_PROV,
-					 g_param_spec_object ("provider", NULL, NULL, 
+					 g_param_spec_object ("provider", NULL,
+							      "Database provider which created the object", 
 							      GDA_TYPE_SERVER_PROVIDER,
 							      G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
 	g_object_class_install_property (object_class, PROP_SPEC_FILE,
-					 g_param_spec_string ("spec-filename", NULL, NULL, 
+					 g_param_spec_string ("spec-filename", NULL,
+							      "XML file which contains the object's data structure", 
 							      NULL, G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
 	g_object_class_install_property (object_class, PROP_OP_TYPE,
-					 g_param_spec_int ("op-type", NULL, NULL, 
+					 g_param_spec_int ("op-type", NULL, "Type of operation to be done", 
 							   0, GDA_SERVER_OPERATION_LAST - 1, 
 							   0, G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
 }
@@ -501,6 +519,7 @@ sequence_add_item (GdaServerOperation *op, Node *node)
 }
 
 static void xml_validity_error_func (void *ctx, const char *msg, ...);
+static gboolean use_xml_spec (GdaServerOperation *op, xmlDocPtr doc, const gchar *xmlfile);
 
 static void
 gda_server_operation_set_property (GObject *object,
@@ -549,7 +568,6 @@ gda_server_operation_set_property (GObject *object,
 			const gchar *xmlfile;
 			static GHashTable *doc_hash = NULL; /* key = file name, value = xmlDocPtr */
 
-			xmlDtdPtr old_dtd = NULL;
 			xmlfile = g_value_get_string (value);
 			if (!xmlfile)
 				return;
@@ -572,52 +590,8 @@ gda_server_operation_set_property (GObject *object,
 			
 			doc = xmlParseFile (xmlfile);
 			if (doc) {
-				/* doc validation */
-				xmlValidCtxtPtr validc;
-				int xmlcheck;
-				
-				validc = g_new0 (xmlValidCtxt, 1);
-				validc->userData = op;
-				validc->error = xml_validity_error_func;
-				validc->warning = NULL;
-				
-				xmlcheck = xmlDoValidityCheckingDefaultValue;
-				xmlDoValidityCheckingDefaultValue = 1;
-				
-				/* replace the DTD with ours */
-				if (gda_server_op_dtd) {
-					old_dtd = doc->intSubset;
-					doc->intSubset = gda_server_op_dtd;
-				}
-#ifndef G_OS_WIN32
-				if (doc->intSubset && !xmlValidateDocument (validc, doc)) {
-					gchar *str;
-					
-					if (gda_server_op_dtd)
-						doc->intSubset = old_dtd;
-					xmlFreeDoc (doc);
-					g_free (validc);
-					str = g_object_get_data (G_OBJECT (op), "xmlerror");
-					if (str) {
-						g_warning (_("GdaServerOperation: file '%s' does not conform to DTD:\n%s"),
-							   xmlfile, str);
-						g_free (str);
-						g_object_set_data (G_OBJECT (op), "xmlerror", NULL);
-					}
-					else
-						g_warning (_("GdaServerOperation: file '%s' does not conform to DTD"),
-							   xmlfile);
-					
-					xmlDoValidityCheckingDefaultValue = xmlcheck;
+				if (!use_xml_spec (op, doc, xmlfile))
 					return;
-				}
-#endif
-				
-				xmlDoValidityCheckingDefaultValue = xmlcheck;
-				g_free (validc);
-				if (gda_server_op_dtd)
-					doc->intSubset = old_dtd;
-				op->priv->xml_spec_doc = doc;
 				g_hash_table_insert (doc_hash, g_strdup (xmlfile), doc);
 			}
 			else {
@@ -643,7 +617,7 @@ gda_server_operation_set_property (GObject *object,
 		}
 	}
 }
-
+	
 static void
 gda_server_operation_get_property (GObject *object,
 				   guint param_id,
@@ -668,6 +642,73 @@ gda_server_operation_get_property (GObject *object,
 	}
 }
 
+/*
+ * Steals @doc (it is freed if necessary)
+ */
+static gboolean
+use_xml_spec (GdaServerOperation *op, xmlDocPtr doc, const gchar *xmlfile)
+{
+	/* doc validation */
+	xmlValidCtxtPtr validc;
+	int xmlcheck;
+	xmlDtdPtr old_dtd = NULL;
+	
+	validc = g_new0 (xmlValidCtxt, 1);
+	validc->userData = op;
+	validc->error = xml_validity_error_func;
+	validc->warning = NULL;
+	
+	xmlcheck = xmlDoValidityCheckingDefaultValue;
+	xmlDoValidityCheckingDefaultValue = 1;
+	
+	/* replace the DTD with ours */
+	if (gda_server_op_dtd) {
+		old_dtd = doc->intSubset;
+		doc->intSubset = gda_server_op_dtd;
+	}
+#ifndef G_OS_WIN32
+	if (doc->intSubset && !xmlValidateDocument (validc, doc)) {
+		gchar *str;
+		
+		if (gda_server_op_dtd)
+			doc->intSubset = old_dtd;
+		xmlFreeDoc (doc);
+		g_free (validc);
+		str = g_object_get_data (G_OBJECT (op), "xmlerror");
+		if (str) {
+			if (xmlfile)
+				g_warning (_("GdaServerOperation: file '%s' does not conform to DTD:\n%s"),
+					   xmlfile, str);
+			else
+				g_warning (_("GdaServerOperation specification does not conform to DTD:\n%s"),
+					   str);
+			g_free (str);
+			g_object_set_data (G_OBJECT (op), "xmlerror", NULL);
+		}
+		else {
+			if (xmlfile)
+				g_warning (_("GdaServerOperation: file '%s' does not conform to DTD"),
+					   xmlfile);
+			else
+				g_warning (_("GdaServerOperation specification does not conform to DTD\n"));
+		}
+		
+		xmlDoValidityCheckingDefaultValue = xmlcheck;
+		xmlFreeDoc (doc);
+		return FALSE;
+	}
+#endif
+	
+	xmlDoValidityCheckingDefaultValue = xmlcheck;
+	g_free (validc);
+	if (gda_server_op_dtd)
+		doc->intSubset = old_dtd;
+	op->priv->xml_spec_doc = doc;
+
+	return TRUE;
+}
+
+
 /*
  * function called when an error occurred during the document validation
  */
@@ -935,8 +976,44 @@ load_xml_spec (GdaServerOperation *op, xmlNodePtr specnode, const gchar *root, G
 	return retlist;
 }
 
+/*
+ * @xml_spec: the specifications for the GdaServerOperation object to create as a string
+ * Internal function
+ */
+GdaServerOperation *
+_gda_server_operation_new_from_string (GdaServerOperationType op_type, 
+				       const gchar *xml_spec)
+{
+	GObject *obj;
+	xmlDocPtr doc;
+	GdaServerOperation *op;
+
+	doc = xmlParseMemory (xml_spec, strlen (xml_spec) + 1);
+	if (!doc)
+		return NULL;
+	obj = g_object_new (GDA_TYPE_SERVER_OPERATION, "op-type", op_type, NULL);
+	op = GDA_SERVER_OPERATION (obj);
+	use_xml_spec (op, doc, NULL);
+
+	if (!op->priv->topnodes && op->priv->xml_spec_doc && op->priv->cnc_set && op->priv->prov_set) {
+		/* load XML file */
+		GError *lerror = NULL;
+		op->priv->topnodes = load_xml_spec (op, xmlDocGetRootElement (op->priv->xml_spec_doc), NULL, &lerror);
+		if (!op->priv->topnodes) {
+			g_warning (_("Could not load XML specifications: %s"),
+				   lerror && lerror->message ? lerror->message : _("No detail"));
+			if (lerror)
+				g_error_free (lerror);
+		}
+	}
+
+	return op;
+}
+
+
 /**
  * gda_server_operation_new
+ * @op_type: type of operation
  * @xml_file: a file which has the specifications for the GdaServerOperation object to create
  *
  * IMPORTANT NOTE: Using this funtion is not the recommended way of creating a #GdaServerOperation object, the
@@ -955,7 +1032,8 @@ gda_server_operation_new (GdaServerOperationType op_type, const gchar *xml_file)
 {
 	GObject *obj;
 
-	obj = g_object_new (GDA_TYPE_SERVER_OPERATION, "op-type", op_type, "spec-filename", xml_file, NULL);
+	obj = g_object_new (GDA_TYPE_SERVER_OPERATION, "op-type", op_type, 
+			    "spec-filename", xml_file, NULL);
 #ifdef GDA_DEBUG_NO
 	{
 		g_print ("New GdaServerOperation:\n");
@@ -1132,10 +1210,14 @@ gda_server_operation_op_type_to_string (GdaServerOperationType type)
 		return "DROP_INDEX";
         case GDA_SERVER_OPERATION_RENAME_TABLE:
 		return "RENAME_TABLE";
+        case GDA_SERVER_OPERATION_COMMENT_TABLE:
+		return "COMMENT_TABLE";
         case GDA_SERVER_OPERATION_ADD_COLUMN:
 		return "ADD_COLUMN";
         case GDA_SERVER_OPERATION_DROP_COLUMN:
 		return "DROP_COLUMN";
+        case GDA_SERVER_OPERATION_COMMENT_COLUMN:
+		return "COMMENT_COLUMN";
 	case GDA_SERVER_OPERATION_CREATE_VIEW:
 		return "CREATE_VIEW";
 	case GDA_SERVER_OPERATION_DROP_VIEW:
@@ -1811,33 +1893,25 @@ gda_server_operation_del_item_from_sequence (GdaServerOperation *op, const gchar
 	return FALSE;
 }
 
-/**
- * gda_server_operation_get_value_at
+/*
+ * real_gda_server_operation_get_value_at
  * @op: a #GdaServerOperation object
- * @path_format: a complete path to a node (starting with "/")
- * @...: arguments to use with @path_format to make a complete path
+ * @path: a complete path to a node (starting with "/")
  *
- * Get the value for the node at the path formed using @path_format and ... (the rules are the same as
- * for g_strdup_printf())
+ * Get the value for the node at the @path path
  *
  * Returns: a constant #GValue if a value has been defined, or %NULL if the value is undefined or
  * if the @path is not defined or @path does not hold any value.
  */
-const GValue *
-gda_server_operation_get_value_at (GdaServerOperation *op, const gchar *path_format, ...)
+static const GValue *
+real_gda_server_operation_get_value_at (GdaServerOperation *op, const gchar *path)
 {
 	const GValue *value = NULL;
 	GdaServerOperationNode *node_info;
-	gchar *path;
-	va_list args;
 
 	g_return_val_if_fail (GDA_IS_SERVER_OPERATION (op), NULL);
 	g_return_val_if_fail (op->priv, NULL);
-
-	/* build path */
-	va_start (args, path_format);
-	path = g_strdup_vprintf (path_format, args);
-	va_end (args);
+	g_return_val_if_fail (path && *path, NULL);
 
 	/* use path */
 	node_info = gda_server_operation_get_node_info (op, path);
@@ -1880,10 +1954,90 @@ gda_server_operation_get_value_at (GdaServerOperation *op, const gchar *path_for
 		}		
 	}
 
-	g_free (path);
 	return value;
 }
 
+/**
+ * gda_server_operation_get_value_at
+ * @op: a #GdaServerOperation object
+ * @path_format: a complete path to a node (starting with "/")
+ * @...: arguments to use with @path_format to make a complete path
+ *
+ * Get the value for the node at the path formed using @path_format and ... (the rules are the same as
+ * for g_strdup_printf())
+ *
+ * Returns: a constant #GValue if a value has been defined, or %NULL if the value is undefined or
+ * if the @path is not defined or @path does not hold any value.
+ */
+const GValue *
+gda_server_operation_get_value_at (GdaServerOperation *op, const gchar *path_format, ...)
+{
+	const GValue *value = NULL;
+	gchar *path;
+	va_list args;
+
+	g_return_val_if_fail (GDA_IS_SERVER_OPERATION (op), NULL);
+	g_return_val_if_fail (op->priv, NULL);
+
+	/* build path */
+	va_start (args, path_format);
+	path = g_strdup_vprintf (path_format, args);
+	va_end (args);
+
+	value = real_gda_server_operation_get_value_at (op, path);
+	g_free (path);
+
+	return value;
+}
+
+/**
+ * gda_server_operation_get_sql_identifier_at
+ * @op: a #GdaServerOperation object
+ * @cnc: a #GdaConnection, or %NULL
+ * @prov: a #GdaServerProvider, or %NULL
+ * @path_format: a complete path to a node (starting with "/")
+ * @...: arguments to use with @path_format to make a complete path
+ *
+ * This method is similar to gda_server_operation_get_value_at(), but for SQL identifiers: a new string
+ * is returned instead of a #GValue. Also the returned string is assumed to represents an SQL identifier
+ * and will correctly be quoted to be used with @cnc, or @prov if @cnc is %NULL (a generic quoting rule
+ * will be applied if both are %NULL).
+ *
+ * Returns: a new string, or %NULL if the value is undefined or
+ * if the @path is not defined or @path does not hold any value, or if the value held is not a string
+ * (in that last case a warning is shown).
+ *
+ * Since: 4.0.3
+ */
+gchar *
+gda_server_operation_get_sql_identifier_at (GdaServerOperation *op, GdaConnection *cnc, GdaServerProvider *prov,
+					    const gchar *path_format, ...)
+{
+	const GValue *value = NULL;
+	gchar *path;
+	va_list args;
+	GdaConnectionOptions cncoptions = 0;
+
+	g_return_val_if_fail (GDA_IS_SERVER_OPERATION (op), NULL);
+
+	/* build path */
+	va_start (args, path_format);
+	path = g_strdup_vprintf (path_format, args);
+	va_end (args);
+
+	value = real_gda_server_operation_get_value_at (op, path);
+	g_free (path);
+
+	if (!value || (G_VALUE_TYPE (value) == GDA_TYPE_NULL))
+		return NULL;
+	g_return_val_if_fail (G_VALUE_TYPE (value) == G_TYPE_STRING, NULL);
+
+	if (cnc)
+		g_object_get (G_OBJECT (cnc), "options", &cncoptions, NULL);
+	return gda_sql_identifier_quote (g_value_get_string (value), cnc, prov, FALSE,
+					 cncoptions & GDA_CONNECTION_OPTIONS_SQL_IDENTIFIERS_CASE_SENSITIVE);
+}
+
 /**
  * gda_server_operation_set_value_at
  * @op: a #GdaServerOperation object
@@ -2081,7 +2235,7 @@ gda_server_operation_set_value_at (GdaServerOperation *op, const gchar *value, G
  * Tells if all the required values in @op have been defined.
  *
  * if @xml_file is not %NULL, the validity of @op is tested against that specification, 
- * and not againts the current @op's specification.
+ * and not against the current @op's specification.
  *
  * Returns: TRUE if @op is valid
  */
