@@ -75,7 +75,7 @@ gda_vconnection_data_model_init (GdaVconnectionDataModel *cnc, GdaVconnectionDat
 	cnc->priv = g_new (GdaVconnectionDataModelPrivate, 1);
 	cnc->priv->table_data_list = NULL;
 
-	g_object_set (G_OBJECT (cnc), "cnc_string", "_IS_VIRTUAL=TRUE", NULL);
+	g_object_set (G_OBJECT (cnc), "cnc-string", "_IS_VIRTUAL=TRUE", NULL);
 }
 
 static void
@@ -248,12 +248,12 @@ gda_vconnection_data_model_add (GdaVconnectionDataModel *cnc, GdaVconnectionData
 	cnc->priv->table_data_list = g_slist_append (cnc->priv->table_data_list, td);
 
 	/* actually create the virtual table in @cnc */
-	prov = (GdaVirtualProvider *) gda_connection_get_provider_obj (GDA_CONNECTION (cnc));
+	prov = (GdaVirtualProvider *) gda_connection_get_provider (GDA_CONNECTION (cnc));
 	str = g_strdup_printf ("CREATE VIRTUAL TABLE %s USING %s ('%s')", table_name, G_OBJECT_TYPE_NAME (prov), td->unique_name);
 	rc = sqlite3_exec (scnc->connection, str, NULL, 0, &zErrMsg);
 	g_free (str);
 	if (rc != SQLITE_OK) {
-		g_set_error (error, 0, 0, g_strdup (zErrMsg));
+		g_set_error (error, 0, 0, "%s", zErrMsg);
 		sqlite3_free (zErrMsg);
 		gda_vconnection_data_model_table_data_free (td);
 		cnc->priv->table_data_list = g_slist_remove (cnc->priv->table_data_list, td);
@@ -301,18 +301,18 @@ gda_vconnection_data_model_remove (GdaVconnectionDataModel *cnc, const gchar *ta
 
 	td = gda_vconnection_get_table_data_by_name (cnc, table_name);
 	if (!td) {
-		g_set_error (error, 0, 0,
+		g_set_error (error, 0, 0, "%s",
 			     _("Table to remove not found"));
 		return FALSE;
 	}
 	
-	prov = (GdaVirtualProvider *) gda_connection_get_provider_obj (GDA_CONNECTION (cnc));
+	prov = (GdaVirtualProvider *) gda_connection_get_provider (GDA_CONNECTION (cnc));
 	str = g_strdup_printf ("DROP TABLE %s", td->table_name);
 	rc = sqlite3_exec (scnc->connection, str, NULL, 0, &zErrMsg);
 	g_free (str);
 
 	if (rc != SQLITE_OK) {
-		g_set_error (error, 0, 0, g_strdup (zErrMsg));
+		g_set_error (error, 0, 0, "%s", zErrMsg);
 		sqlite3_free (zErrMsg);
 		return FALSE;
 	}
@@ -381,14 +381,18 @@ gda_vconnection_data_model_get_table_name (GdaVconnectionDataModel *cnc, GdaData
 /**
  * gda_vconnection_data_model_foreach
  * @cnc: a #GdaVconnectionDataModel connection
- * @func: a #GdaVConnectionDataModelFunc function pointer
- * @data: data to pass to @cunc calls
+ * @func: a #GdaVconnectionDataModelFunc function pointer
+ * @data: data to pass to @func calls
  *
- * Call @func for each #GdaDataModel represented as a table in @cnc.
+ * Call @func for each table in @cnc. 
+ *
+ * Warning: @func will be called for any table present in @cnc even if no data
+ * model represents the contents of the table (which means the 1st argument of @func
+ * may be %NULL)
  */
 void
 gda_vconnection_data_model_foreach (GdaVconnectionDataModel *cnc, 
-				    GdaVConnectionDataModelFunc func, gpointer data)
+				    GdaVconnectionDataModelFunc func, gpointer data)
 {
 	GSList *list, *next;
 	g_return_if_fail (GDA_IS_VCONNECTION_DATA_MODEL (cnc));
@@ -401,8 +405,7 @@ gda_vconnection_data_model_foreach (GdaVconnectionDataModel *cnc,
 	while (list) {
 		GdaVConnectionTableData *td = (GdaVConnectionTableData*) list->data;
 		next = list->next;
-		if (td->spec->data_model)
-			func (td->spec->data_model, td->table_name, data);
+		func (td->spec->data_model, td->table_name, data);
 		list = next;
 	}
 }
@@ -448,6 +451,10 @@ gda_vconnection_data_model_table_data_free (GdaVConnectionTableData *td)
 {
 	if (td->real_model)
 		g_object_unref (td->real_model);
+	if (td->columns) {
+		g_list_foreach (td->columns, (GFunc) g_object_unref, NULL);
+		g_list_free (td->columns);
+	}
 	g_free (td->table_name);
 	g_free (td->unique_name);
 	if (td->spec_free_func)
