@@ -630,16 +630,7 @@ gda_mysql_provider_open_connection (GdaServerProvider               *provider,
 					     (compress && ((*compress == 't') || (*compress == 'T'))) ? TRUE : FALSE,
 					     &error);
 	if (!mysql) {
-		GdaConnectionEvent *event_error = gda_connection_event_new (GDA_CONNECTION_EVENT_ERROR);
-		gda_connection_event_set_sqlstate (event_error, _("Unknown"));
-		gda_connection_event_set_description (event_error,
-						      error && error->message ? error->message :
-						      _("No description"));
-		gda_connection_event_set_code (event_error, GDA_CONNECTION_EVENT_CODE_UNKNOWN);
-		gda_connection_event_set_source (event_error, "gda-mysql");
-		gda_connection_add_event (cnc, event_error);
-		g_clear_error (&error);
-
+		_gda_mysql_make_error (cnc, mysql, NULL, NULL);
 		return FALSE;
 	}
 
@@ -806,12 +797,12 @@ gda_mysql_provider_supports_operation (GdaServerProvider       *provider,
         case GDA_SERVER_OPERATION_CREATE_TABLE:
         case GDA_SERVER_OPERATION_DROP_TABLE:
         case GDA_SERVER_OPERATION_RENAME_TABLE:
-
+        case GDA_SERVER_OPERATION_COMMENT_TABLE:
         case GDA_SERVER_OPERATION_ADD_COLUMN:
-
+	case GDA_SERVER_OPERATION_DROP_COLUMN:
+	case GDA_SERVER_OPERATION_COMMENT_COLUMN:
         case GDA_SERVER_OPERATION_CREATE_INDEX:
         case GDA_SERVER_OPERATION_DROP_INDEX:
-
         case GDA_SERVER_OPERATION_CREATE_VIEW:
         case GDA_SERVER_OPERATION_DROP_VIEW:
 		return TRUE;
@@ -824,7 +815,7 @@ gda_mysql_provider_supports_operation (GdaServerProvider       *provider,
  * Create operation request
  *
  * Creates a #GdaServerOperation. The following code is generic and should only be changed
- * if some further initialization is required, or if operation's contents is dependant on @cnc
+ * if some further initialization is required, or if operation's contents is dependent on @cnc
  */
 static GdaServerOperation *
 gda_mysql_provider_create_operation (GdaServerProvider       *provider,
@@ -921,12 +912,18 @@ gda_mysql_provider_render_operation (GdaServerProvider   *provider,
         case GDA_SERVER_OPERATION_RENAME_TABLE:
 		sql = gda_mysql_render_RENAME_TABLE (provider, cnc, op, error);
 		break;
+        case GDA_SERVER_OPERATION_COMMENT_TABLE:
+		sql = gda_mysql_render_COMMENT_TABLE (provider, cnc, op, error);
+		break;
         case GDA_SERVER_OPERATION_ADD_COLUMN:
 		sql = gda_mysql_render_ADD_COLUMN (provider, cnc, op, error);
 		break;
 	case GDA_SERVER_OPERATION_DROP_COLUMN:
 		sql = gda_mysql_render_DROP_COLUMN (provider, cnc, op, error);
 		break;
+	case GDA_SERVER_OPERATION_COMMENT_COLUMN:
+		sql = gda_mysql_render_COMMENT_COLUMN (provider, cnc, op, error);
+		break;
         case GDA_SERVER_OPERATION_CREATE_INDEX:
 		sql = gda_mysql_render_CREATE_INDEX (provider, cnc, op, error);
 		break;
@@ -937,8 +934,8 @@ gda_mysql_provider_render_operation (GdaServerProvider   *provider,
 		sql = gda_mysql_render_CREATE_VIEW (provider, cnc, op, error);
 		break;
         case GDA_SERVER_OPERATION_DROP_VIEW:
-                sql = gda_mysql_render_DROP_VIEW (provider, cnc, op, error);
-                break;
+		sql = gda_mysql_render_DROP_VIEW (provider, cnc, op, error);
+		break;
         default:
                 g_assert_not_reached ();
         }
@@ -1378,7 +1375,6 @@ gda_mysql_provider_create_parser (GdaServerProvider  *provider,
  * 
  * This method renders a #GdaStatement into its SQL representation.
  */
-static gchar *mysql_render_function (GdaSqlFunction *func, GdaSqlRenderingContext *context, GError **error);
 static gchar *
 gda_mysql_provider_statement_to_sql (GdaServerProvider    *provider,
 				     GdaConnection        *cnc,
@@ -1400,9 +1396,6 @@ gda_mysql_provider_statement_to_sql (GdaServerProvider    *provider,
         memset (&context, 0, sizeof (context));
         context.params = params;
         context.flags = flags;
-	context.render_function = (GdaSqlRenderingFunc) mysql_render_function; /* don't put any space between function name
-										* and the opening parenthesis, see
-										* http://blog.152.org/2009/12/mysql-error-1305-function-xxx-does-not.html */
 	if (cnc) {
 		context.cnc = cnc;
 		context.provider = gda_connection_get_provider (cnc);
@@ -1424,40 +1417,6 @@ gda_mysql_provider_statement_to_sql (GdaServerProvider    *provider,
         return str;
 }
 
-static gchar *
-mysql_render_function (GdaSqlFunction *func, GdaSqlRenderingContext *context, GError **error)
-{
-	GString *string;
-	gchar *str;
-	GSList *list;
-
-	g_return_val_if_fail (func, NULL);
-	g_return_val_if_fail (GDA_SQL_ANY_PART (func)->type == GDA_SQL_ANY_SQL_FUNCTION, NULL);
-
-	/* can't have: func->function_name == NULL */
-	if (!gda_sql_any_part_check_structure (GDA_SQL_ANY_PART (func), error)) return NULL;
-
-	string = g_string_new (func->function_name);
-	g_string_append_c (string, '(');
-	for (list = func->args_list; list; list = list->next) {
-		if (list != func->args_list)
-			g_string_append (string, ", ");
-		str = context->render_expr (list->data, context, NULL, NULL, error);
-		if (!str) goto err;
-		g_string_append (string, str);
-		g_free (str);
-	}
-	g_string_append_c (string, ')');
-
-	str = string->str;
-	g_string_free (string, FALSE);
-	return str;
-
- err:
-	g_string_free (string, TRUE);
-	return NULL;
-}
-
 static GdaMysqlPStmt *
 real_prepare (GdaServerProvider *provider, GdaConnection *cnc, GdaStatement *stmt, GError **error)
 {
@@ -2233,12 +2192,12 @@ my_remove_quotes (gchar *str)
 
         total = strlen (str);
         if (str[total-1] == delim) {
-		/* string is correclty terminated */
+		/* string is correctly terminated */
 		g_memmove (str, str+1, total-2);
 		total -=2;
 	}
 	else {
-		/* string is _not_ correclty terminated */
+		/* string is _not_ correctly terminated */
 		g_memmove (str, str+1, total-1);
 		total -=1;
 	}
